import os
import cPickle as pickle
from net import Net
from urlparse import urlparse
from datetime import datetime
import time
from ordereddict import OrderedDict
import simplejson as json
import urllib2
import socket
import sys

class ApiTimeout(Exception):
	pass

class EmptyCredentials(Exception):
    pass

class ApiError(Exception):
    pass
	
class TimeZoneError(Exception):
	pass

class TsApi:

	_api_url = None
	_username = None
	_password = None
	_cache_path = None
	_cookie = None
	_net = None
	params = None
	playlist = None
	eventplan = None
	recorder = None
	
	def __init__(self, username, password, api_url, cache_path):
		
		if username == "" or password == "":
			raise EmptyCredentials()

		self._username = username
		self._password = password
		self._api_url = api_url
		
		self._cache_path = cache_path
		self._cookie = os.path.join(cache_path, "cookie.dat")
		
		if not os.path.exists(cache_path):
			os.makedirs(cache_path)
		
		self._net = Net(cookie_file=self._cookie)
		self.params = self._get_data("params")
		self.playlist = self._get_data("playlist")
		self.eventplan = self._get_data("eventplan")
		self.recorder = self._get_data("recorder")
		
	def log(self, msg):
		print "TEAMSTREAM: " + msg
		
	def _is_cookie_expired(self):
		domain = urlparse(self._api_url).hostname
		for k, v in self._net.get_cookies().iteritems():
			if k == domain:
				return v["/"]["PHPSESSID"].is_expired()
		
		return True
		
	def _auth(self):
		url = self._api_url + "auth"
		
		if self._is_cookie_expired():
			print "expired"
			form_data = {	"username": self._username,
							"password": self._password }
			try:
				html = self._net.http_POST(url, form_data)
				if self._is_cookie_expired():
					self.log("Server said its %s, but you said its %s" % (html._response.info().getheader('Date'), time.strftime("%a, %d %b %Y %H:%M:%S GMT", time.gmtime())))
					raise TimeZoneError()
			except urllib2.URLError, e:
				if isinstance(e.reason, socket.timeout):
					raise ApiTimeout()
				else:
					raise
			if self._net.get_cookies():
				self.log("Saving cookie to " + self._cookie)
				self._net.save_cookies(self._cookie)
			else:
				self.log("No cookie found")
			api = json.loads(html.content)
			if api['success'] == "true":
				return True
			else:
				raise ApiError(api['error'])
		else:
			return True

	def check_running_event(self, id):
		today = datetime.now().strftime("%d.%m.%Y")
		now = datetime.now().strftime("%H:%M")
		now = int(now.split(":")[0]) * 60 + int(now.split(":")[1])
		try:
			for event in self.eventplan[today]:
				if event["channel"] == id:
					start, end = event["time"].split(" - ")
					start = int(start.split(":")[0]) * 60 + int(start.split(":")[1])
					end = int(end.split(":")[0]) * 60 + int(end.split(":")[1])
					if start <= now <= end:
						return event["event"]
		except:
			pass
		return None
			
	def get_params_by_id(self, id):
		for channel in self.playlist["Stream Box"]:
			if id == channel["channel_id"]:
				return channel["rtmp"], channel["playpath"]
		for channel in self.playlist["TV Box SD"]:
			if id == channel["title"]:
				return channel["rtmp"], channel["playpath"]
				
		return "NoPlayPathDefined"
			
	def _get_data(self, method):
		url = self._api_url + method
		
		cache = os.path.join(self._cache_path, method + ".cache")
		create_cache = False
		time_now = datetime.utcnow()
		
		if not os.path.exists(cache):
			self.log("%s doesn't exist - creating ..." % (method + ".cache"))
			create_cache = True
		else:
			time_file = datetime.utcfromtimestamp(os.path.getmtime(cache))
			time_now = int(time_now.strftime("%Y%m%d"))
			time_file = int(time_file.strftime("%Y%m%d"))
			if time_now > time_file:
				self.log("Recreating %s ..." % (method + ".cache"))
				create_cache = True
			
		if create_cache:
			self._auth()
			try:
				html = self._net.http_GET(url).content
			except urllib2.URLError, e:
				if isinstance(e.reason, socket.timeout):
					raise ApiTimeout()
				else:
					raise
			api = json.loads(html, object_pairs_hook=OrderedDict)
			if api['success'] == 'true':
				pickle.dump(api['data'], open(cache, 'w'))
				return api['data']
			else:
				raise ApiError(api['error'])
		else:
			return pickle.load(open(cache, 'r'))