import re
import datetime
import xbmcaddon
import sys, traceback
import urllib2
import os
import logging
from lib import Addon
ts_addon	= Addon("plugin.video.teamstream", sys.argv)
__cwd__ = ts_addon.get_path()

logging.basicConfig(format="%(message)s")

sys.path.insert(0,xbmc.translatePath(os.path.join(__cwd__, "lib")))
sys.path.insert(0,xbmc.translatePath(os.path.join(__cwd__, "lib", "simplejson")))

from tsapi import TsApi, ApiTimeout, EmptyCredentials, ApiError, TimeZoneError
from hoerzu import HoerzuEPG

cache 		= os.path.join(xbmc.translatePath("special://temp"), 'teamstream').decode('utf-8')
img_path	= os.path.join(ts_addon.get_path(), 'resources', 'images').decode('utf-8')

username 	= ts_addon.get_setting("login")
password 	= ts_addon.get_setting("password")
api_url 	= ts_addon.get_setting("api_url")

api = None

days = ["Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag", "Sonntag" ]

#functions
def download_image(url, img):
	try:
		print "Downloading image " + url
		response = urllib2.urlopen(url)
		the_page = response.read()
		f = open(img, 'wb')
		f.write(the_page)
		f.close()
		return img
	except:
		exc_type, exc_value, exc_traceback = sys.exc_info()
		traceback.print_exc()
		traceback.print_tb(exc_traceback)
		print "Image download failed: " + url	
		return ""

def get_image(image, url=False, download=False):
	if url:
		image_url = image
		match = re.search(r"[^/?]*\.[^/?]*(\?.*)?$", image, re.DOTALL)
		if match:
			image = match.group(0)
		else:
			result = ""
	else:
		image = image.replace(" ", "_")
		
	image = os.path.join(img_path, image)
	
	if os.path.exists(image):
		return image
	else:
		print "File does not exist " + image
		if download:
			return download_image(image_url, image)
		return ""

#menu stuff
def show_main_menu():
	for box in api.playlist:
		img = get_image(box+".png")
		infolabel =  { "title": box }
		output_queries = { "mode": "box", "box": box }
		ts_addon.add_directory(output_queries, infolabel, img=img)
		
	ts_addon.add_directory({"mode":"recorder"}, {"title":"TS Recorder"}, img=get_image("Recorder.png"))
	ts_addon.add_directory({"mode":"eventplan"}, {"title":"Eventplan"}, img=get_image("Eventplan.png"))
	ts_addon.end_of_directory()
	
def show_box(box):
	epg = HoerzuEPG()
	for channel in api.playlist[box]:
		
		img = get_image(channel["image"],True)
		if box == "Stream Box":
			event = api.check_running_event(channel["channel_id"])
		else:
			event = None
		
		if event is not None:
			title = channel["title"] + " - [EVENT] " + event
		else:
			epg_string = epg.get_epg_channel(channel["epg"])	
			if epg_string is not None:
				title = channel["title"] + " - " + epg_string
			else:
				title = channel["title"] + " - Keine EPG-Daten vorhanden"
		resolved = "%s swfUrl=%s pageUrl=%s playpath=%s swfVfy=true live=true" % (channel['rtmp'], api.params["player"], "http://www.teamstream.to/", channel["playpath"])
		infolabel = { "title": title }
		ts_addon.add_video_item({}, infolabel, resolved=resolved, img=img)
	ts_addon.end_of_directory()
	
def show_recorder_index():
	for category in api.recorder:
		infolabel =  { "title": category }
		output_queries = { "mode": "recorder_category", "recorder": category }
		ts_addon.add_directory(output_queries, infolabel)
	ts_addon.end_of_directory()
	
def show_recorder_category(category):
	category = category.decode( "utf-8")
	for item in api.recorder[category]:
		resolved = "%s swfUrl=%s pageUrl=%s playpath=mp4:%s swfVfy=true" % (item["rtmp"], item["player"], "http://www.teamstream.to/", item["playpath"])
		infolabel = { "title": item["title"] }
		ts_addon.add_video_item({}, infolabel, resolved=resolved )
	ts_addon.end_of_directory()
	
def show_eventplan_index():
	for date in api.eventplan:
		day, month, year = date.split(".")
		date_now = datetime.datetime.now().strftime("%d.%m.%Y")
		weekday = datetime.datetime(int(year), int(month), int(day)).weekday()
		title = "%s - %s" % (date, days[weekday])	
		if date == date_now:
			title += " (heute)"	
		img = get_image(days[weekday]+".png")
		infolabel =  { "title": title }
		output_queries = { "mode": "eventplan_day", "eventplan": date }
		ts_addon.add_directory(output_queries, infolabel, img=img)
	ts_addon.end_of_directory()


def show_eventplan_day(date):
	for event in api.eventplan[date]:
		rtmp, playpath = api.get_params_by_id(event["channel"])
		resolved = "%s swfUrl=%s pageUrl=%s playpath=%s swfVfy=true live=true" % (rtmp, api.params["player"], "http://www.teamstream.to/", playpath)
		title = "%s - [%s]: %s" % (event["time"], event["channel"], event["event"])
		img = get_image(event["img"],True, True)
		infolabel =  { "title": title }
		ts_addon.add_video_item({}, infolabel, resolved=resolved, img=img )
	ts_addon.end_of_directory()


# Addon start
def main():
	global api
	input_queries = ts_addon.queries
	
	if input_queries['mode'] == "rebuild_cache":
		for root, dirs, files in os.walk(cache, topdown=False):
			for name in files:
				os.remove(os.path.join(root, name))
		api = TsApi(username, password, api_url, cache)
		ts_addon.show_small_popup("Teamstream", "Cache wurde erfolgreich erneuert")
	else:
		api = TsApi(username, password, api_url, cache)
	
		if input_queries['mode'] == 'main':
			api.log("v" + ts_addon.get_version())
			show_main_menu()
		elif input_queries['mode'] == 'box':
			show_box(input_queries['box'])
		elif input_queries['mode'] == "recorder":
			show_recorder_index()
		elif input_queries['mode'] == "recorder_category":
			show_recorder_category(input_queries['recorder'])
		elif input_queries['mode'] == "eventplan":
			show_eventplan_index()
		elif input_queries['mode'] == "eventplan_day":
			show_eventplan_day(input_queries['eventplan'])
		

try:		
	main()
except EmptyCredentials, ex:
	ts_addon.show_error_dialog( [">> Anmeldedaten unvollstaendig <<", "\nBitte trage deinen Login im folgendem Dialog ein."])
	ts_addon.show_settings()
	ts_addon.add_item({}, {"title":"Teamsteam.to"}, resolved="plugin://plugin.video.teamstream/")
	ts_addon.end_of_directory()
except TimeZoneError:
	ts_addon.show_error_dialog( [">> API Error: time out of joint <<", "\nDeine Zeit/Zeitzonen-Einstellungen stimmen wahr-","\ncheinlich nicht oder weichen zu stark vom Server ab."])
except ApiTimeout:
	ts_addon.show_error_dialog( [">> API Error: timeout <<", "\nDie TS-API ist momentan leider nicht erreichbar. Bitte","\ninformiere dich im Chat oder versuche es spaeter nochmal!"])
except ApiError, ex:
	ts_addon.show_error_dialog( [">> API Error: %s <<" % ex])
except Exception, ex:
	msg = ">> %s <<" % ex
	logging.exception(msg)
	msg = [ msg, "\nBitte poste diese Fehlermeldung und dein xbmc.log ins", "\nSupport-Forum." ]
	ts_addon.show_error_dialog(msg)
	pass

		

	